import { FileOneView } from "./FileOneView";
import React, { useState } from "react";
import { MdDownload, MdOutlineDelete } from "react-icons/md";

import FileInput from "./FileInput";
import { FileViewInput } from "../types";
import { ChangingArrow } from "./ChangingArrow";
export function FileView({
  generateFile,
  grammar,
  getFile,
  fileToGrammar,
  deleteFile,
  deleteFilesFromGrammar,
}: FileViewInput) {
  const [xml, setXml] = useState(false);
  const [more, setMore] = useState(false);
  const [fileInput, setFileInput] = useState(false);
  const [file, setFile] = useState<File | null>(null);
  const [text, setText] = useState("");
  const [saveGenFile, setSaveGenFile] = useState(false);
  const [rows, setRows] = useState(10);
  const fileAdd = (
    file: string,
    xml: boolean | null,
    save: boolean,
    diff: boolean
  ) => {
    fileToGrammar(file, xml, grammar.id, save, diff);
  };
  const calculateRows = (xml: boolean) => {
    const value = xml ? grammar.xmlGen : grammar.jsgfGen;
    console.log(value);
    if (value === undefined) {
      setRows(7);
      return;
    }
    const lineBreaks = (value.match(/\n/g) || []).length;
    console.log(lineBreaks);
    setRows(Math.min(lineBreaks + 2, 15));
  };
  return (
    <>
      <p>
        <input
          id="gensave"
          type="checkbox"
          defaultChecked={saveGenFile}
          onClick={() => setSaveGenFile(!saveGenFile)}
        ></input>
        <label className="smallText" htmlFor="gensave">
          save
        </label>{" "}
        <button
          onClick={() => {
            generateFile(xml, grammar.id, saveGenFile);
            calculateRows(xml);
          }}
        >
          GENERATE {xml ? "XML" : "JSGF"}{" "}
        </button>{" "}
        <a
          href={`${
            window.location.hostname === "localhost"
              ? "http://localhost:8080"
              : ""
          }/api/files/grammar/${grammar.id}/generated?format=${
            xml ? "xml" : "jsgf"
          }&download=${true}`}
        >
          <MdDownload />
        </a>{" "}
        <button
          onClick={() => {
            setXml(!xml);
            calculateRows(!xml);
          }}
        >
          {xml ? "JSGF" : "XML"}
        </button>{" "}
      </p>
      {(xml ? grammar.xmlGen : grammar.jsgfGen) && (
        <p>
          <textarea
            className="transparent big"
            readOnly={true}
            rows={rows}
            cols={50}
            value={xml ? grammar.xmlGen : grammar.jsgfGen}
          />
        </p>
      )}
      <button onClick={() => setFileInput(!fileInput)}>
        {fileInput ? "CANCEL" : "ADD FILE"}
      </button>
      {"  "}
      {fileInput && (
        <FileInput
          fileAdd={fileAdd}
          file={file}
          setFile={setFile}
          setText={setText}
          text={text}
          newDiff={true}
        />
      )}
      <div className="mediumSize">
        {`${grammar.files.length} files`}
        <MdOutlineDelete onClick={() => deleteFilesFromGrammar(grammar.id)} />
        <ChangingArrow more={more} setMore={setMore} />
      </div>

      {more && (
        <div className="table">
          {grammar.files.map((file) => (
            <FileOneView
              key={file.id}
              file={file}
              getFile={getFile}
              deleteFile={deleteFile}
              grammar={grammar}
            />
          ))}
        </div>
      )}
    </>
  );
}
