package com.jakubu9333.grammarManager.api;

import com.fasterxml.jackson.annotation.JsonView;
import com.jakubu9333.grammarManager.model.FileHolder;
import com.jakubu9333.grammarManager.model.Grammar;
import com.jakubu9333.grammarManager.model.View;
import com.jakubu9333.grammarManager.service.FileService;
import org.javatuples.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author Jakub Uhlarik
 */

@RestController
@CrossOrigin(origins = "http://localhost:3000")
@RequestMapping(path = "api/files")
public class FilesControler {
    private final FileService fileService;

    @Autowired
    public FilesControler(FileService fileService) {
        this.fileService = fileService;
    }


    @GetMapping("grammar/{grammarId}")
    public ResponseEntity<String> fileFromGrammar( @PathVariable Long grammarId, @RequestParam(defaultValue = "false") boolean download, HttpServletResponse response) {
        MediaType typeOfFile;

        String fileName;
        FileHolder result = fileService.getFileFromGrammar(grammarId);
        if (result.isXml()) {
            typeOfFile = MediaType.TEXT_PLAIN;

            fileName = grammarId + "_g.jsgf";
        } else {
            typeOfFile = MediaType.APPLICATION_XML;
            fileName = grammarId + "_g.grxml";
        }
        if (download) {
            response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
        }
        return ResponseEntity.status(HttpStatus.OK).contentType(typeOfFile).body(result.getText());
    }


    @JsonView(View.RuleWithoutGrammar.class)
    @PostMapping("")
    public ResponseEntity<Grammar> grammarFromFile(@RequestBody String file, @RequestParam(required = false) String format) {
        if (format == null) {
            Grammar grammar = fileService.addGrammarFromFile(file, null).getValue0();
            return new ResponseEntity<>(grammar, HttpStatus.CREATED);
        }
        Boolean xml;
        switch (format) {
            case "jsgf" -> xml = false;

            case "xml" -> xml = true;
            case "u" -> xml = null;
            default -> throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "invalid file format");
        }
        Grammar grammar = fileService.addGrammarFromFile(file, xml).getValue0();
        return new ResponseEntity<>(grammar, HttpStatus.CREATED);
    }
    //#todo file from grammar format

}
