package com.jakubu9333.grammarManager.dao;

import lombok.NoArgsConstructor;
import org.basex.api.client.ClientSession;
import org.basex.core.Command;
import org.basex.core.cmd.Add;
import org.basex.core.cmd.XQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;

/**
 * @author Jakub Uhlarik
 */
@NoArgsConstructor
public class XmlDataDao {

    /**
     * baseX server starts on default port 1984
     */
    private ClientSession session;


    public static String xQueryRule(String id) {
        return "descendant::*:rule[(@id = \"" + id + "\")]";
    }

    private static String xQueruGrammar(String root) {
        return "descendant::*:grammar[(@root = \"" + root + "\")]";
    }

    @Autowired
    public XmlDataDao(ClientSession session) {
        this.session = session;
    }

    private String executeCommand(Command command) {
        try {
            return session.execute(command);
        } catch (IOException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    public String getByRoot(String root) {
        return executeCommand(new XQuery(xQueruGrammar(root)));
    }

    public String getRuleById(String id) {
        return executeCommand(new XQuery(xQueryRule(id)));
    }


    public void write(String xml) {
        executeCommand(new Add("xd", xml));
    }
}
