package com.jakubu9333.grammarManager.model;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonView;

import lombok.Getter;
import lombok.Setter;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PostUpdate;
import javax.persistence.SequenceGenerator;

import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Jakub Uhlarik
 */
@Entity
@Getter
@Setter
public class Grammar {

    @Id
    @JsonView(View.All.class)
    @SequenceGenerator(name = "grammar_sequence", allocationSize = 1, sequenceName = "grammar_sequence")
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "grammar_sequence")
    private Long id;

    @JsonView(View.All.class)
    @Column(columnDefinition = "text")
    private String uri;

    // @JsonView(View.All.class)
    private LocalDateTime updateTime;


    @OneToOne(cascade = CascadeType.ALL)
    @JsonView(View.All.class)
    private FileHolder file;


    @OneToMany(cascade = CascadeType.ALL,mappedBy = "grammar")
    @JsonView(View.RuleWithoutGrammar.class)
    private Set<RuleModel> rules;

    @JsonView(View.RuleWithoutGrammar.class)
    private String rootSymbol;


    public Grammar() {
        this(null, null, null);
    }

    public Grammar(String uri, Set<RuleModel> rules, String rootSymbol) {
        this(null, uri, rules, rootSymbol);
    }
    public Grammar(String uri ){
        this(null, uri, null, null);
    }


    @JsonCreator
    public Grammar(Long id, String uri, Set<RuleModel> rules, String rootSymbol) {
        this.rootSymbol = rootSymbol;
        this.uri = uri;
        if (rules == null) {
            this.rules = new HashSet<>();
            return;
        }
        this.rules = rules;

    }

    @PostUpdate
    public void preUpdate() {
        this.updateTime = LocalDateTime.now();
    }
}
