package com.jakubu9333.grammarManager;

import com.jakubu9333.grammarManager.model.Grammar;
import com.jakubu9333.grammarManager.model.RuleModel;
import com.jakubu9333.grammarManager.service.FileService;
import com.jakubu9333.grammarManager.service.GrammarService;

import com.jakubu9333.grammarManager.service.RuleService;
import org.javatuples.Pair;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.web.server.ResponseStatusException;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;



/**
 * @author Jakub Uhlarik
 */
@SpringBootTest
public class FileGrammarServiceTest {

    @Autowired
    public GrammarService grammarService;

    @Autowired
    public FileService fileService;
    @Autowired
    public RuleService ruleService;

    @AfterEach
    public void tearDown() {
        grammarService.deleteAll();
    }

    @Test
    public void jsgfReadTest() {
        boolean xml = false;
        try {
            fileService.addGrammarFromFile("", xml);
            assert false;
        } catch (Exception ignored) {

        }

        String file1 = """
                #JSGF V1.0;
                grammar first;
                <rule1> = smile;
                //root;
                <rule2> = cry|sis;
                                
                               
                          
                """;
        Grammar firstGrammar = fileService.addGrammarFromFile(file1, xml).getValue0();
        assert firstGrammar.getUri().equals("first");
        assert firstGrammar.getRules().size() == 2;
        assert firstGrammar.getRules().equals(new HashSet<>(List.of(new RuleModel("rule1", "smile"), new RuleModel("rule2", "cry|sis"))));

        String file2 = """
                #JSGF V1.0;
                grammar dms;
                ddsd=sfdsfd;
                grammar xfs;
                                
                """;
        boolean error = false;
        try {
            fileService.addGrammarFromFile(file2, xml);
        } catch (Exception e) {
            error = true;
        }
        assert error;
        String file3 = """
                                
                grammar dms;
                //root;
                ddsd=sfdsfd;
                 //root;
                ddssfsd=sfdsfd;
                                
                """;
        error = false;
        try {
            fileService.addGrammarFromFile(file3, xml);
        } catch (ResponseStatusException e) {
            error = true;
        }
        assert error;
    }



    @Test
    public void xmlReadTest() {
        boolean xml = true;
        boolean er = false;
        try {
            fileService.addGrammarFromFile("", xml);
        } catch (ResponseStatusException e) {
            er = true;
        }
        assert er;

        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                             <item>sm <ruleref special= "NULL"  />hah</item>
                            
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        Grammar firstGrammar = fileService.addGrammarFromFile(file1, xml).getValue0();
        assert firstGrammar.getUri().equals("first");
        String file2 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                             <item><ruleref special= "NULL"  /></item>
                            <item>x<ruleref special= "VOID"  /></item>
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        fileService.addGrammarFromFile(file2, xml);


        String fileRoot = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                           root="rule1"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        Grammar rootGr = fileService.addGrammarFromFile(fileRoot, xml).getValue0();
        assert rootGr.getRootSymbol() != null;
        String file3 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="second"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                             <item>sm <ruleref special= "NULL"  />hah</item>
                            
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    cry sis  <ruleref uri= "#xd"  />
                                </rule>
                            </grammar>
                          
                """;
        Grammar second = fileService.addGrammarFromFile(file3, xml).getValue0();
        assert second.getUri().equals("second");
    }

    @Test
    public void testErrorMessages() {


        String file = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="second"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>"123"
                            </item>
                            <item>"smile "<ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                     
                                <rule id="rule2">
                                    <item>"cry"</item>
                                </rule>
                            </grammar>
                          
                """;
        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            </rule>
                            </grammar>
                """;

        List<String> files = new ArrayList<>();

        files.add(file);
        files.add(file1);

        String file2 = """
                             <?xml version="1.0"?>
                             <rule id="2"> </rule>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              </grammar>
                """;
        files.add(file2);


        String file3 = """
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"><item> "ddd" </item>
                              </rule>
                              <one-of>
                              </one-of>
                              </grammar>
                """;
        files.add(file3);

        String file4 = """
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"bubub"
                              </one-of>
                              </rule>
                             </item>
                              </grammar>
                """;
        files.add(file4);


        String file5 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"cry"
                             </item>
                              </rule>
                              </one-of>
                             
                              </grammar>
                """;
        files.add(file5);

        String file6 = """
                            <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"><item>"smile"</item>
                              </one-of>
                              </rule>
                             
                              </grammar>
                """;
        files.add(file6);

        String file7 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"xd"</item>
                              </one-of>
                              </rule>
                            <item>"outside"</item>
                              </grammar>
                """;

        files.add(file7);
        String file8 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"ff</item>
                              </one-of>
                              </rule>
                              <rule id="5>  <one-of><item>"ls</item>
                              </one-of>
                    
                              </grammar>
                """;

        files.add(file8);


        for (String s : files) {

            testErrors(s);
        }

    }

    private void testErrors(String file) {

        try {
            fileService.addGrammarFromFile(file.trim(), true);
            assert false;
        } catch (ResponseStatusException ignored) {

        }

    }


    @Test
    public void testComment() {
        String file1 = """
                #JSGF  V1.0;
                grammar first;
                /* dsg = xdcko
                drgd
                dgdgdg
                dgdgd
                dgdgd
                rdrgdgdgd
                */
                <rule3>  =/* xd   */ smiling;
                /* xd  fsdfsfsfsfsf
                sfsf
                sfsdfdsf
                sdfdsfsd
                dfsdfsdf
                sfsdfdsfsdf
                dsfsdfdsfs */ <rule4> = xdd;
                /* dfsfsfssfs */  <rule5> = xdgd;
                //<rule1> = smile;
                //root;
                <rule2> = ddggd;
                """;

        Grammar grammar = fileService.addGrammarFromFile(file1, false).getValue0();
        RuleModel rule2 = new RuleModel( "rule2", "ddggd", false);
        RuleModel rule3 = new RuleModel("rule3", "smiling", false);
        RuleModel rule4 = new RuleModel("rule4", "xdd", false);
        RuleModel rule5 = new RuleModel( "rule5", "xdgd", false);
        assert grammar.getRules().size() == 4;


        assert grammar.getRules().equals(new HashSet<>(List.of(rule2, rule4, rule3, rule5)));


        String file2 = """
                 #JSGF v 1.0;
                 grammar first;
                dsg = xdcko */
                 """;
        try {
            fileService.addGrammarFromFile(file2, false);
            assert false;
        } catch (Exception ignored) {

        }
    }






    @Test
    public void testFileSaving() {


        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                              xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="rule1"> <item>smile</item>
                                </rule>
                                <rule id="rule2">
                              <one-of>
                              <item>cry</item>
                              <item>sis</item>
                              </one-of>
                              </rule>
                              </grammar>
                """;

        String file2 = """
                #JSGF V1.0;
                                
                grammar first1;
                <rule1> = smile;
                <rule2> = cry|sis;
                """;
        Grammar grammarFromJsgf = fileService.addGrammarFromFile(file2, false).getValue0();
        Pair<Grammar, Boolean> resultFromXml = fileService.addGrammarFromFile(file1, true);
        Grammar grammarFromXml = resultFromXml.getValue0();
        assert grammarFromXml.getRules().stream().toList().get(1).getRightSide().trim().equals("<item>smile</item>");

        String file2diff = """
                #JSGF V1.0;
                                
                grammar first12;
                <rule1> = smile;
                <rule2> = crfy|sis;
                """;
        Set<RuleModel> ruleSet = new HashSet<>();
        RuleModel rule1 = new RuleModel(null, "rule1", "smile", false);
        RuleModel rule2 = new RuleModel(null, "rule2","cry|sis", false);
        ruleSet.add(rule1);
        ruleSet.add(rule2);
        Grammar grammarFromObj = new Grammar("firstObj", ruleSet, null);
        grammarFromObj = grammarService.processAddGrammar(grammarFromObj);
        Long id = grammarFromObj.getId();

    }

    @Test
    public void testUnknownSource() {
        String file = """
                <?xml version="1.0"?>
                <grammar version="1.0"
                xml:base="xd"
                \t\t\t\t root="sde"
                 xmlns="http://www.w3.org/2001/06/grammar">
                <rule id="p">
                <one-of>
                <item>d
                </item>
                <item>pred<ruleref uri=              "#xd"                  />zad</item>
                </one-of>
                </rule>
                \t<rule id="sde">
                \t\t<item> mam rad zvuk <ruleref uri = "#mirozbirka" />  zvuk jabloní</item>
                \t</rule>
                </grammar>""";

        fileService.addGrammarFromFile(file, null);
        String file2diff = """
                                
                                
                                
                                
                                
                     #JSGF V1.0;
                                
                grammar first12;
                <rule1> = smile;
                <rule2> = crfy|sis;
                """;
        fileService.addGrammarFromFile(file2diff, null);


        String badDiff = """
                                
                                
                                
                                
                                
                     #JGF V1.0;
                                
                grammar first12;
                <rule1> = smile;
                <rule2> = crfy|sis;
                """;
        try {
            fileService.addGrammarFromFile(badDiff, null);
            assert false;
        } catch (Exception ignored) {

        }

    }


}
