package com.jakubu9333.grammarManager;

import com.jakubu9333.grammarManager.model.FileHolder;
import com.jakubu9333.grammarManager.model.Grammar;
import com.jakubu9333.grammarManager.service.FileService;
import org.javatuples.Pair;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.is;
import static org.mockito.ArgumentMatchers.anyBoolean;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.header;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * @author Jakub Uhlarik
 */

@SpringBootTest
@AutoConfigureMockMvc
public class FilesControllerTest {

    private static final String API_FILES_URL = "/api/files";

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private FileService fileService;



    @Test
    public void testGrammarFromFileWithoutFormatParameter() throws Exception {
        Grammar grammar = new Grammar("rthretr");
        when(fileService.addGrammarFromFile(anyString(), eq(null))).thenReturn(new Pair<>(grammar, true));

        mockMvc.perform(post(API_FILES_URL + "/")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"content\":\"file-content\"}"))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id").value(grammar.getId()));
    }

    @Test
    public void testGrammarFromFileUnk() throws Exception {
        Grammar grammar = new Grammar("ergtrhtrh");
        when(fileService.addGrammarFromFile(anyString(), eq(null))).thenReturn(new Pair<>(grammar, true));

        mockMvc.perform(post(API_FILES_URL + "/?format=u")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"content\":\"file-content\"}"))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id").value(grammar.getId()))
                .andExpect(jsonPath("$.uri").value(grammar.getUri()));
    }

    @Test
    public void testGrammarFromFileWithJsgfFormatParameter() throws Exception {
        Grammar grammar = new Grammar("ergerge");
        when(fileService.addGrammarFromFile(anyString(), eq(false))).thenReturn(new Pair<>(grammar, true));

        mockMvc.perform(post(API_FILES_URL + "/?format=jsgf")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"content\":\"file-content\"}"))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id").value(grammar.getId()))
                .andExpect(jsonPath("$.uri").value(grammar.getUri()));
    }

    @Test
    public void testGrammarFromFileWithXmlFormatParameter() throws Exception {
        Grammar grammar = new Grammar();
        when(fileService.addGrammarFromFile(anyString(), eq(true))).thenReturn(new Pair<>(grammar, true));

        mockMvc.perform(post(API_FILES_URL + "/?format=xml")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"content\":\"file-content\"}"))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id").value(grammar.getId()));
    }

    @Test
    public void testGrammarFromFileWithInvalidFormatParameter() throws Exception {
        mockMvc.perform(post(API_FILES_URL + "/?format=invalid")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"content\":\"file-content\"}"))
                .andExpect(status().isBadRequest());
    }



}
